"use strict";
var __asyncValues = (this && this.__asyncValues) || function (o) {
    if (!Symbol.asyncIterator) throw new TypeError("Symbol.asyncIterator is not defined.");
    var m = o[Symbol.asyncIterator], i;
    return m ? m.call(o) : (o = typeof __values === "function" ? __values(o) : o[Symbol.iterator](), i = {}, verb("next"), verb("throw"), verb("return"), i[Symbol.asyncIterator] = function () { return this; }, i);
    function verb(n) { i[n] = o[n] && function (v) { return new Promise(function (resolve, reject) { v = o[n](v), settle(resolve, reject, v.done, v.value); }); }; }
    function settle(resolve, reject, d, v) { Promise.resolve(v).then(function(v) { resolve({ value: v, done: d }); }, reject); }
};
var __await = (this && this.__await) || function (v) { return this instanceof __await ? (this.v = v, this) : new __await(v); }
var __asyncDelegator = (this && this.__asyncDelegator) || function (o) {
    var i, p;
    return i = {}, verb("next"), verb("throw", function (e) { throw e; }), verb("return"), i[Symbol.iterator] = function () { return this; }, i;
    function verb(n, f) { i[n] = o[n] ? function (v) { return (p = !p) ? { value: __await(o[n](v)), done: n === "return" } : f ? f(v) : v; } : f; }
};
var __asyncGenerator = (this && this.__asyncGenerator) || function (thisArg, _arguments, generator) {
    if (!Symbol.asyncIterator) throw new TypeError("Symbol.asyncIterator is not defined.");
    var g = generator.apply(thisArg, _arguments || []), i, q = [];
    return i = {}, verb("next"), verb("throw"), verb("return"), i[Symbol.asyncIterator] = function () { return this; }, i;
    function verb(n) { if (g[n]) i[n] = function (v) { return new Promise(function (a, b) { q.push([n, v, a, b]) > 1 || resume(n, v); }); }; }
    function resume(n, v) { try { step(g[n](v)); } catch (e) { settle(q[0][3], e); } }
    function step(r) { r.value instanceof __await ? Promise.resolve(r.value.v).then(fulfill, reject) : settle(q[0][2], r); }
    function fulfill(value) { resume("next", value); }
    function reject(value) { resume("throw", value); }
    function settle(f, v) { if (f(v), q.shift(), q.length) resume(q[0][0], q[0][1]); }
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.which = exports.filter = exports.traverse = void 0;
const fs_extra_1 = require("fs-extra");
const path_1 = require("path");
const child_process_1 = require("child_process");
const windows_1 = require("./windows");
const { platform, env } = process;
/**
 * Traverse File-system
 *
 * @param path - Root path to traverse
 */
function traverse(path) {
    return __asyncGenerator(this, arguments, function* traverse_1() {
        if (path === '/' && platform === 'win32') {
            for (const disk of (0, windows_1.getPartition)())
                yield __await(yield* __asyncDelegator(__asyncValues(traverse(disk))));
            return yield __await(void 0);
        }
        for (let name of yield __await((0, fs_extra_1.readdir)(path))) {
            name = (0, path_1.join)(path, name);
            yield yield __await(name);
            try {
                if ((yield __await((0, fs_extra_1.stat)(name))).isDirectory())
                    yield __await(yield* __asyncDelegator(__asyncValues(traverse(name))));
            }
            catch (error) {
                switch (error.code) {
                    case 'ENOENT':
                    case 'EPERM':
                    case 'EBUSY':
                    case 'ELOOP':
                    case 'UNKNOWN':
                        continue;
                }
                throw error;
            }
        }
    });
}
exports.traverse = traverse;
/**
 * Iterator filter
 *
 * @param iterator
 * @param pattern  String pattern to match
 * @param count    Result count
 */
function filter(iterator, pattern, count = Infinity) {
    return __asyncGenerator(this, arguments, function* filter_1() {
        var _a, e_1, _b, _c;
        let index = 0;
        count = ~~count || Infinity;
        if (pattern && !(pattern instanceof RegExp))
            pattern = RegExp(pattern + '', 'i');
        try {
            for (var _d = true, iterator_1 = __asyncValues(iterator), iterator_1_1; iterator_1_1 = yield __await(iterator_1.next()), _a = iterator_1_1.done, !_a;) {
                _c = iterator_1_1.value;
                _d = false;
                try {
                    const item = _c;
                    if (!pattern || pattern.test(item))
                        if (index++ < count)
                            yield yield __await(item);
                        else
                            break;
                }
                finally {
                    _d = true;
                }
            }
        }
        catch (e_1_1) { e_1 = { error: e_1_1 }; }
        finally {
            try {
                if (!_d && !_a && (_b = iterator_1.return)) yield __await(_b.call(iterator_1));
            }
            finally { if (e_1) throw e_1.error; }
        }
    });
}
exports.filter = filter;
function Shell_which(name) {
    try {
        return ((0, child_process_1.execSync)(`which ${name}`) + '').trim();
    }
    catch (error) {
        if (!('' + error.stdout + error.stderr).trim())
            return '';
    }
}
const MacAppPath = ['/Applications', `${env.HOME}/Applications`].filter(fs_extra_1.existsSync);
/**
 * @param name - Name (without extension name) of a executable file
 *
 * @return First matched path of a command
 */
async function which(name) {
    var _a, e_2, _b, _c, _d, e_3, _e, _f, _g, e_4, _h, _j;
    switch (platform) {
        case 'win32':
            for (const root of (0, windows_1.getAppFolder)())
                try {
                    for (var _k = true, _l = (e_2 = void 0, __asyncValues(filter(traverse(root), `\\\\${name}\\.exe$`, 1))), _m; _m = await _l.next(), _a = _m.done, !_a;) {
                        _c = _m.value;
                        _k = false;
                        try {
                            const file = _c;
                            return file;
                        }
                        finally {
                            _k = true;
                        }
                    }
                }
                catch (e_2_1) { e_2 = { error: e_2_1 }; }
                finally {
                    try {
                        if (!_k && !_a && (_b = _l.return)) await _b.call(_l);
                    }
                    finally { if (e_2) throw e_2.error; }
                }
            break;
        case 'darwin': {
            const path = Shell_which(name);
            if (path)
                return path;
            for (const root of MacAppPath)
                try {
                    for (var _o = true, _p = (e_3 = void 0, __asyncValues(filter(traverse(root), `\\.app\\/Contents\\/MacOS\\/(\\w+\\W)?${name}$`, 1))), _q; _q = await _p.next(), _d = _q.done, !_d;) {
                        _f = _q.value;
                        _o = false;
                        try {
                            const file = _f;
                            return file;
                        }
                        finally {
                            _o = true;
                        }
                    }
                }
                catch (e_3_1) { e_3 = { error: e_3_1 }; }
                finally {
                    try {
                        if (!_o && !_d && (_e = _p.return)) await _e.call(_p);
                    }
                    finally { if (e_3) throw e_3.error; }
                }
            break;
        }
        default: {
            const path = Shell_which(name);
            if (path)
                return path;
            try {
                for (var _r = true, _s = __asyncValues(filter(traverse('/opt'), `${name}$`, 1)), _t; _t = await _s.next(), _g = _t.done, !_g;) {
                    _j = _t.value;
                    _r = false;
                    try {
                        const file = _j;
                        return file;
                    }
                    finally {
                        _r = true;
                    }
                }
            }
            catch (e_4_1) { e_4 = { error: e_4_1 }; }
            finally {
                try {
                    if (!_r && !_g && (_h = _s.return)) await _h.call(_s);
                }
                finally { if (e_4) throw e_4.error; }
            }
        }
    }
    return '';
}
exports.which = which;
